// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assert.hpp"

namespace Dawn {
  enum TicTacToeTileState {
    TIC_TAC_TOE_EMPTY,
    TIC_TAC_TOE_NOUGHT,
    TIC_TAC_TOE_CROSS
  };

  /**
   * Determine the winner of the given board.
   * 
   * @param board Tic tac toe board.
   * @param winningCombo The output winning combo (if any).
   * @return The winning player, or EMPTY if no winner is present.
   */
  enum TicTacToeTileState ticTacToeDetermineWinner(
    const std::map<uint8_t, enum TicTacToeTileState> board,
    std::vector<uint8_t> *winningCombo
  );

  /**
   * Returns true if the tic tac toe game is over. Will also consider ties as a
   * game over state.
   * 
   * @param board Board to check if game has ended.
   * @return True if game is over, otherwise false.
   */
  bool_t ticTacToeIsGameOver(
    const std::map<uint8_t, enum TicTacToeTileState> board
  );

  /**
   * Returns the score / value of a given board for the given player. Mostly
   * used by the AI to determine whether a given board is better or worse than
   * any other.
   * 
   * @param board Board to get the score of.
   * @param player Player to get the score for.
   * @return The weighted score of this board.
   */
  int32_t ticTacToeGetBoardScore(
    std::map<uint8_t, enum TicTacToeTileState> board,
    enum TicTacToeTileState player
  );

  /**
   * Returns which cell should be used by the given player for their AI as the
   * best move for them.
   * 
   * @param board Tic tac toe board.
   * @param player Player to get the AI move for.
   * @return The recommended cell to fill.
   */
  uint8_t ticTacToeGetAiMove(
    std::map<uint8_t, enum TicTacToeTileState> board,
    enum TicTacToeTileState player
  );
}