// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "scene/components/entity/EntityFaction.hpp"

namespace Dawn {
  enum EntityAttackState {
    ENTITY_ATTACK_STATE_RAMP_UP,
    ENTITY_ATTACK_STATE_ACTIVE,
    ENTITY_ATTACK_STATE_RAMP_DOWN,
    ENTITY_ATTACK_STATE_COOLDOWN,
    ENTITY_ATTACK_STATE_NOTHING
  };

  class EntityAttackBase : public SceneItemComponent {
    protected:
      float_t attackTime = -1;

    public:
      StateProperty<bool_t> interrupted;
      StateProperty<enum EntityAttackState> state;
      StateEvent<> onAttackRampUpStart;
      StateEvent<> onAttackRampUpEnd;
      StateEvent<> onAttackActiveStart;
      StateEvent<> onAttackActiveEnd;
      StateEvent<> onAttackRampDownStart;
      StateEvent<> onAttackRampDownEnd;
      StateEvent<> onAttackCooldownStart;
      StateEvent<> onAttackCooldownEnd;
      StateEvent<> onAttackInterrupted;

      EntityAttackBase(SceneItem* item);
      void onStart() override;

      /**
       * Returns the duration of the attack ramp up. The ramp up is how long it
       * takes before the attack starts doing damage. For example, this would be
       * in a sword attack, how long it takes the player to cock their arm back
       * before swinging.
       * 
       * @return The duration of the attack ramp up.
       */
      virtual float_t getAttackRampUpDuration() = 0;

      /**
       * Returns the duration of the attack active. The active is how long the
       * attack is doing damage. For example, with a sword this would be the
       * time that the swing itself is occuring.
       * 
       * @return The duration of the attack active.
       */
      virtual float_t getAttackActiveDuration() = 0;

      /**
       * Returns the duration of the attack ramp down. The ramp down is how long
       * it takes after the attack is done doing damage, but before the cooldown
       * begins. Example, if swing a heavy sword, this is how long after the 
       * sword hits the ground but before the player has picked the sword back
       * up.
       * 
       * @return The duration of the ramp down.
       */
      virtual float_t getAttackRampDownDuration() = 0;

      /**
       * Returns the duration of the attack cooldown. The cooldown is how long
       * it takes before the attack can be used again. For example, if someone
       * shoots a gun, this would be how long it takes to cycle a bullet in a 
       * rifle.
       * 
       * @return The duration of the attack cooldown.
       */
      virtual float_t getAttackCooldownDuration() = 0;

      /**
       * Decide whether this attack can be interrupted or not. If this returns
       * false, then the attack cannot be interrupted.
       * 
       * @return True if the attack can be interrupted, false otherwise.
       */
      virtual bool_t isInterruptable() = 0;

      /**
       * Returns true if this attack can be performed, false otherwise.
       * 
       * @return True if this attack can be performed, false otherwise.
       */
      virtual bool_t canAttack() = 0;

      /**
       * Requests this attack to be performed.
       */
      void attack();

      /**
       * Interrupts the current attack.
       */
      void interrupt();

      /**
       * Returns true if the entity is currently attacking, false otherwise.
       * 
       * @return True if the entity is currently attacking, false otherwise.
       */
      bool_t isAttacking();
  };
}